
--[[
-- Indexing Sample Mappings Utilities
-- for Renoise API Ver.4
-- uprime22
--]]

-- loading functions for unicode
-- I use a 'chopup' function by jpjacobs in this module.  Thank You !
-- http://forum.luahub.com/index.php?topic=3522.0
require ('unicode_utilities')


--  For one-to-one injective,sample_name to sample_mapping.
-- Return the table in whitch a word-key have a table
-- 	,{['note']=note_number
--	,['velo']=velocity_number
--	,['inst']=instrument_index}
-- num_letters:max number of letters for making key-strings
-- 	from sample names. 
-- init_lst:initial table
function name2note_velo_idx (inst_idx,num_letters,init_lst)
  local maps = init_lst
  for i,v in pairs (renoise.song().instruments[inst_idx].samples) do
    local _name = v.name
    local choped_name = chopup(_name) -- from 'unicode_utilities'
    local len = #choped_name
    if len > num_letters
    then len = num_letters
    end
      
    -- 'ho ge' -> 'ho'
    local n,_name = cut_utf(sub_utf(_name,1,len),' ') -- separator: space
    local note = v.sample_mapping.note_range[1]
    local velo = v.sample_mapping.velocity_range
    if velo[1]==0 and velo[2]==127 
    then
      velo = 255
    else
      velo = velo[2]
      end
    local tbl ={}
    tbl['note'] = note
    tbl['velo'] = velo
    tbl['inst'] = inst_idx
    if _name ~= nil then
      maps[_name] = tbl
    end
  end
  return maps
end



-- tbl={1,2};tbl2={1,2};test ={}
-- test[tbl]='a'
-- test[tbl2] --> nil
-- set_meta_eq(tbl,tbl2)
-- set_meta_index(test)
-- test[tbl2] --> a
function meta_eq (tbl1, tbl2)
  if type(tbl1) =='table' and type(tbl2) =='table'
  then
    for i,v in pairs(tbl1) do
      if tbl2[i] ~= v
      then return false
      end
    end
    return true
  else return (tbl1 == tbl2)
  end
end

function set_meta_eq (...)  
  for i=1,select('#',...) do -- (#...)͑ʖڂB
    local t = select(i,...) -- ̕ϐɑ鎖ŁAl擪̈ɐ
    setmetatable(t,{__eq = meta_eq}) 
  end
end
  
function get_with_eq (tbl,index)
  for i,v in pairs(tbl) do
    if i == index then
      return v
    end
  end
  return nil
end

function set_meta_index (...)
  for i=1,select('#',...) do
    local t = select(i,...)
    setmetatable(t,{__index = get_with_eq})
  end
end



-- note_val2string (48) -> 'C-4'
function note_val2string (value)
  local str
  if value == 120 then
    return 'OFF'
  elseif value == 121 then
    return '---'
  elseif value >=0 and value <=119 then
    local note ={
      'C-','C#','D-','D#','E-','F-',
      'F#','G-','G#','A-','A#','B-'}
    local oct = math.modf(value/12)
    local ni = value - (oct*12)
    str =
    note[ni+1]..tostring(oct)
    return str
  else return nil
  end
end


-- get_val_from_name ('hogexx-2.5yy','xx','yy') -> -2.5
function get_val_from_name (name ,tag1,...)
  if type(name)=='string' and type(tag1)=='string'
  then
    local tag2 =...
    if tag2 ==nil then tag2 ="" else end
    local val =
    tonumber(
      string.match(
        tostring(
          string.match(name,tag1.."[-+]?%d+%.?%d*"..(tag2))),"%-?%d+%.?%d*"))
    return val
  else return nil
  end
end


-- For one-to-one injective,note_columns to sample infomations.
-- This return the list in that a key is
--  string of a note_column's note,instrument,veloctiy combination
--  ,ex.'C-4007F'.
-- If note_range is {0,127},the key is 'C-400..'.
function note_column2sample_info(inst_idx,init_lst,num_letters)
  local inst_max = #(renoise.song().instruments)
  if inst_idx > inst_max then
    return init_lst
  end
  local tbl = init_lst
  local maps =
  renoise.song().instruments[inst_idx].sample_mappings[1]
  local spl_lst =
  renoise.song().instruments[inst_idx].samples
  for i = 1,#spl_lst do
--    local spl = spl_lst[i].sample_index
    local name = spl_lst[i].name
    local choped_name = chopup(name) -- from 'unicode_utilities'
    local len = #choped_name
    if type(num_letters)=='number' and len > num_letters
    then len = num_letters
    end
    -- 'ho ge' -> 'ho'
    local x,key = cut_utf(sub_utf(name,1,len),' ') -- from 'unicode_utilities'     
    -- string.format('%02X',tostring(128)) -> 'FF'
    local inst = string.format('%02X',tostring(inst_idx -1))
    local nt = spl_lst[i].sample_mapping.note_range
    local vl = spl_lst[i].sample_mapping.velocity_range
    if vl[1]==0 and vl[2]==127 then vl = {255,255}
    end
    for n = nt[1],nt[2] do
      for v = vl[1],vl[2] do
        v = string.format('%02X',tostring(v))
        if v == 'FF' then v = '..' end
        local id =
        note_val2string(n)..inst..v
        tbl[id] = {['index']= i, ['name']= name,['key']= key}
      end
    end
  end
  return tbl
end

-- Add value infomations to the list
function add_values_info(note_column2sample_info_list,tag1,...)
  local tbl={}
  for k,t in pairs(note_column2sample_info_list) do
    local val = get_val_from_name (t.name ,tag1,...)
    t[tag1] = val
    tbl[k] = t
  end
  return tbl
end



