require 'globals'
require 'indexing_mappings'
---------------------------------------
-- 現在編集位置の各indexを返す
function pos2 ()
  local ep,s,p,t,l,c
  ep = renoise.song().transport.edit_pos
  s = ep.sequence
  l = ep.line  
  p = renoise.song().sequencer:pattern(s)
  t = renoise.song().selected_track_index
  c = renoise.song().selected_note_column_index
  -- ?? Debugger sometimes says,index ==0.
  if c == nil or c == 0 then
    c = 1
  end
  return s,p,t,l,c
end


-- inst_idxにて、note_column2sample_info_lstをUpdateする。
-- TrackDelayの更新も追加。
function update_sample_etc_lst (inst_idx)
  note_column2sample_info_lst =
  note_column2sample_info(inst_idx,note_column2sample_info_lst,max_number_letters)
  note_column2sample_info_lst =  
  add_values_info(note_column2sample_info_lst,tags[1],tags[2])
  track_delay =
  renoise.song().selected_track.output_delay
end


-- Renoise NoteColumnObject から、SampleNameを割り出し、
-- tag1などに囲まれた数字があればそれを返す
-- volume_option==true なら、Volume:7F時の値を返す。
function get_val_only (note_column,tag1,...)
  local info = {}
  local nt_str = note_column.note_string
  if nt_str =='OFF' or nt_str =='---'then
    return nil
  end  
  local inst_str = note_column.instrument_string
  local vol_str = note_column.volume_string
  -- volume_option 追加。7FのMappingにValueを一致させるか否か。
  if vol_str == '..' or volume_option == true then
    vol_str = '7F'
  end
  local tbl = note_column2sample_info_lst[nt_str..inst_str..'..']
  if type(tbl) == 'table' then
    info = tbl
  else tbl = note_column2sample_info_lst[nt_str..inst_str..vol_str]
    if type(tbl) == 'table' then
      info = tbl
    end
  end
  return info[tag1]
end


-- Renoise NoteColumnObject から、SampleNameを割り出し、
-- tag1などに囲まれた数字があればそれを返す。
-- Sampleの名前やIndexなども色々返すように変更。
function get_tagged_val_etc_from_note (note_column,tag1,...)
  local info = {}
  local nt_str = note_column.note_string
  if nt_str =='OFF' or nt_str =='---'then
    return nil
  end
  local inst_str = note_column.instrument_string
  local inst_idx = note_column.instrument_value +1
  if inst_idx >=256 
  then inst_idx = 
    renoise.song().selected_instrument_index
  else end  
  local vol_str = note_column.volume_string
  if vol_str == '..' then
    vol_str = '7F'
  end
  local dly_val = note_column.delay_value
  local id = nt_str..inst_str..'..'
  local tbl = note_column2sample_info_lst[id]
  if type(tbl) == 'table' then
    info = tbl
  else tbl = note_column2sample_info_lst[nt_str..inst_str..vol_str]
    if type(tbl) == 'table' then
      info = tbl
    end
  end
  local name = info.name
  local sample_idx = info.index
  local val = get_val_only (note_column,tag1) -- for volume option
  return val ,name ,sample_idx ,inst_idx ,dly_val
end



-- SampleEditorに移動。
function to_sample_editor (inst_idx,smpl_idx)
  if type(smpl_idx) == 'number' and
    type(inst_idx) == 'number'
  then
    renoise.song().selected_instrument_index = inst_idx
    renoise.song().selected_sample_index = smpl_idx
    renoise.app().window.active_middle_frame =
    smpl_edtr
  else end
end
-- PatternEditorに移動。
function to_pattern_editor ()
  renoise.app().window.active_middle_frame =
  ptn_edtr
end


-- Functions for Renaming etc.

-- nameから、tag1などを付随する数字と共に除去。
function remove_tag_val (name,tag1,...)
  local tag2 =...
  if tag2 ==nil then tag2 ="" else end
    
  local loc1,loc2 =
  string.find(name,tag1.."%-?%+?%d+%.?%d*"..tag2)
  if loc1 ==nil or loc2 ==nil then return name
  else
  local val =
  (name:sub(1,loc1 -1))..(name:sub(loc2 +1))
  return val
  end
end

-- http://lua-users.org/wiki/SimpleRound
function round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end

-- Frame数をミリ秒に変換。
-- SampleRate割り出すのに、SampleEditor上で使われる想定で
-- selected_sample使ってるので、ちょっと注意。
function frame2ms(frame)
  local init_sample_rate =
  renoise.song().selected_sample.sample_buffer.sample_rate
  local ms=
  round(1000*(frame/init_sample_rate),2)
  return ms
end

-- 選択開始点までは、どれくらいのミリ秒か。
-- -符号付。
function selection_start_2_ms_minus ()
  local init_buffer =
  renoise.song().selected_sample.sample_buffer
  local start =
  init_buffer.selection_start
  return -1*frame2ms(start -1)
end

-- 選択したSampleの名前に、与えたタグと値を付加。
function rename_selected_sample_with_tags (val,tag1,...)
  if type(val)~='number' or type(tag1)~='string' or rename_option == false
  then return nil
  else
    local sample =
    renoise.song().selected_sample
    sample.name = remove_tag_val (sample.name,tag1,...)
    sample.name = (sample.name)..tag1..tostring(val)..(...)
    update_sample_etc_lst (renoise.song().selected_instrument_index)
  end
end

-- 選択したSampleの名前から、指定のタグとそれに付随した値を取り除く。
function remove_tags_from_selected_sample (tag1,...)
  if type(tag1)~='string' or rename_option == false
  then return nil
  else
    local sample =
    renoise.song().selected_sample
    sample.name = remove_tag_val (sample.name,tag1,...)
    update_sample_etc_lst (renoise.song().selected_instrument_index)
  end
end


-- Delay値などの計算

-- ミリ秒が、何ラインと何Delay値分かを計算。
function ms2delay(ms)
  local bpm = renoise.song().transport.bpm
  local lpb = renoise.song().transport.lpb
  local mspl = (60*1000)/(bpm*lpb)
  if type(ms)~='number' then ms=0
  else end
  local ln , dly_l = math.modf(ms/mspl)
  local dly = round(dly_l*256)
  if dly ==256 then ln =ln +1 ; dly =0
  else end
  return ln,dly
end

-- 上の結果でDelay値を必ず正値にする。
-- TrackOutputDelayや、DelayValueも、もしあれば、計算に入れる。
function delay2delay(ms,...)
  local ln,dly
  if type(ms) ~='number' then
    ms = 0
  end
  local trk_dly,ini_dly = ...
  
  if type(trk_dly) ~= 'number' or track_delay_option == false
  then trk_dly = 0
  else end
  if type(ini_dly) ~= 'number' or delay_val_option == false
  then ini_dly = 0
  else end
  ln,dly = ms2delay(ms - trk_dly)
  if dly <0
  then ln = ln -1 ; dly = 256 +dly
  else ln =ln ; dly = dly
  end
  
  dly = dly + ini_dly
  local mod = math.modf(dly/256)
  ln = ln + mod
  dly = dly - (mod * 256)
  return ln , dly
end


-- Functions for Nudge

-- 前と次のSequenceのPatternIndexを返す。無けりゃNil。
-- デフォだとエラーが返るんで。
function pre_next_ptn (seq_pos)
  local seq_length = #(renoise.song().sequencer.pattern_sequence)
  local previous_ptn, next_ptn
  if type(seq_pos)=='number' and seq_pos <= seq_length
    and seq_pos >= 1 then
      if seq_pos > 1 and seq_pos < seq_length
      then
        previous_ptn = renoise.song().sequencer:pattern(seq_pos -1)
        next_ptn = renoise.song().sequencer:pattern(seq_pos +1)
      elseif seq_pos >= seq_length and seq_length >1
      then
        previous_ptn = renoise.song().sequencer:pattern(seq_pos -1)
        next_ptn = nil
      elseif seq_pos <= 1 and seq_length >1
      then
        previous_ptn = nil
        next_ptn = renoise.song().sequencer:pattern(seq_pos +1)
      elseif seq_length == 1
      then previous_ptn = nil ; next_ptn = nil
      else end
    else end
  return previous_ptn ,next_ptn
end

-- s_i,l_iからline分移動したときの、
-- 次のSeqencePosとLine位置を返す。
-- Delay値とかはdelay2delayとかで計算するから要らないyo。
function next_seq_line (line,s_i,l_i)
  local s_len = #(renoise.song().sequencer.pattern_sequence)
  if type(s_i)=='number' and s_i <= s_len
    and s_i >= 1 and type(line) =='number' and type(l_i) =='number'
  then
    local p_i = renoise.song().sequencer:pattern(s_i)
    local cp_len = renoise.song().patterns[p_i].number_of_lines
    local pre_ptn,nxt_ptn = pre_next_ptn (s_i)    
    local nxt_s = s_i
    local nxt_l = l_i + line
    if nxt_l <= 0 and l_i ~= 1 then
      nxt_s,nxt_l = next_seq_line(nxt_l-1,s_i,1)
    elseif nxt_l > cp_len and l_i ~= cp_len then
      nxt_s,nxt_l = next_seq_line(nxt_l - cp_len,s_i,cp_len)
    elseif line < 0 and l_i ==1 and pre_ptn ~=nil then
      nxt_s,nxt_l = next_seq_line(line +1,s_i -1,
        renoise.song().patterns[pre_ptn].number_of_lines)
    elseif line > 0 and l_i == cp_len and nxt_ptn ~= nil then
      nxt_s,nxt_l = next_seq_line(line -1,s_i +1,1)
    elseif line < 0 and l_i ==1 and pre_ptn == nil then
      nxt_s ,nxt_l = nil,nil
    elseif line > 0 and l_i == cp_len and nxt_ptn == nil then
      nxt_s ,nxt_l = nil,nil
    elseif line == 0 and l_i >= 1 and l_i <= cp_len then
      nxt_s, nxt_l = s_i, l_i
    else end
    return nxt_s,nxt_l
  else return nil,nil
  end
end

-- もしやNudge済みのやつでは？可能性があるならTrue。
-- 値を反転させて、現Sequenceから飛びでるかで判定。
function already_nudged_maybe (s_i,t_i,l_i,c_i)
  local p_i = renoise.song().sequencer:pattern(s_i)
  local note_column =
  renoise.song().patterns[p_i].tracks[t_i].lines[l_i].note_columns[c_i]
  local val =
  get_val_only (note_column,tags[1],tags[2])
  if note_column.is_empty == false and type(val) =='number' then
    local dly_val = note_column.delay_value
    local l = delay2delay(-val,0,dly_val)
    local n_s = next_seq_line (l,s_i,l_i)
    if n_s ~= s_i then
      return true
    else
      return false
    end
  else return false
  end
end

-- effect column の移動。
function nudge_effect (nxt_s,nxt_p,nxt_t,nxt_l,nxt_e,s,p,t,l,e,effect_option)
  local max_from = renoise.song().tracks[t].max_effect_columns
  local visible = renoise.song().tracks[t].visible_effect_columns
  local max_to = renoise.song().tracks[nxt_t].max_effect_columns
  if nxt_e <= max_from and  visible < nxt_e then
    renoise.song().tracks[t].visible_effect_columns = nxt_e
  end  
  local effect_from ,effect_to
  
  if nxt_s == nil 
    or nxt_e > max_to or e > visible
    or effect_option == false or effect_option == nil
  then return nil
  else
    effect_from=
    renoise.song().patterns[p].tracks[t].lines[l].effect_columns[e]
    
    if effect_from.is_empty == true then
      return
    else
      effect_to =
      renoise.song().patterns[nxt_p].tracks[nxt_t]
      .lines[nxt_l].effect_columns[nxt_e]
      
      if effect_to.is_empty == true then
        effect_to:copy_from(effect_from)
        effect_from:clear()
        nudge_effect (nxt_s,nxt_p,nxt_t,nxt_l,nxt_e +1,s,p,t,l,e +1,effect_option)
      else
        nudge_effect (nxt_s,nxt_p,nxt_t,nxt_l,nxt_e +1,s,p,t,l,e,effect_option)
      end
    end
  end
end


-- 次の位置を求める。次位置が塞がっていれば右隣のカラムへ。
-- Track_optionがTrueなら、次のTrack上で。どうしようもなけりゃNil
function next_pos (line,s_i,p_i,t_i,l_i,c_i,track_option)
  local max_nc =
  renoise.song().tracks[t_i].max_note_columns
  local s_nxt,l_nxt = next_seq_line (line,s_i,l_i)
  local p_nxt,c_nxt
  local t_nxt = t_i
  if track_option == true then
    t_nxt = t_i +1
  end
   
  if s_nxt ~= nil 
  then
    p_nxt = renoise.song().sequencer:pattern(s_nxt)
  else t_nxt = nil
  end

  if p_nxt ~= nil and line ~= 0 and
    renoise.song().patterns[p_nxt].tracks[t_nxt].lines[l_nxt].note_columns[c_i]
    .is_empty == false
    and c_i +1 <=  max_nc
  then
      s_nxt,p_nxt,t_nxt,l_nxt,c_nxt = next_pos (line,s_i,p_i,t_i,l_i,c_i+1,track_option)
  elseif p_nxt ~= nil and line ~= 0 and
    renoise.song().patterns[p_nxt].tracks[t_nxt].lines[l_nxt].note_columns[c_i]
    .is_empty == true
  then
    c_nxt = c_i
  elseif p_nxt ~= nil and line ==0 then
    c_nxt = c_i
  else end
  
  if track_option == true and p_nxt ~= nil and
    renoise.song().patterns[p_nxt].tracks[t_nxt].lines[l_nxt]
    .note_columns[c_nxt].is_empty == false
  then
    s_nxt,p_nxt,t_nxt,l_nxt,c_nxt =
    next_pos (line,s_i,p_i,t_nxt,l_i,c_nxt+1,false)
  else end
  
  return s_nxt,p_nxt,t_nxt,l_nxt,c_nxt
end

-- 指定のカラムをタグに従ってNudge。
function nudge_it (s,p,t,l,c,track_option,direction,effect_option)
  local x = direction
  if type(x) ~='number' then x = 1
  else x = tonumber(x) end
  local s_nxt,p_nxt,t_nxt,l_nxt,c_nxt
  local column_nxt
  local note_column =
  renoise.song().patterns[p].tracks[t].lines[l].note_columns[c]
  local ini_dly = note_column.delay_value
  local ln,dly
  local val = get_val_only (note_column,tags[1],tags[2])
  
  if val ~= nil then
    val = val * x
    ln,dly = delay2delay (val,track_delay,ini_dly)
    s_nxt,p_nxt,t_nxt,l_nxt,c_nxt =
    next_pos (ln,s,p,t,l,c,track_option)
    if s_nxt ~= nil then
    column_nxt =
    renoise.song().patterns[p_nxt].tracks[t_nxt]
    .lines[l_nxt].note_columns[c_nxt]
    else end
    if column_nxt ~= nil
      and (ln ~= 0 or track_option == true)
    then
      column_nxt:copy_from(note_column)
      column_nxt.delay_value = dly
      renoise.song().selected_sequence_index = s_nxt
      renoise.song().selected_track_index = t_nxt
      renoise.song().selected_line_index = l_nxt
      if renoise.song().tracks[t_nxt].visible_note_columns < c_nxt
      then renoise.song().tracks[t_nxt].visible_note_columns = c_nxt
      end
      renoise.song().selected_note_column_index = c_nxt
      -- nudge_effect 追加。
      nudge_effect (s_nxt,p_nxt,t,l_nxt,1,s,p,t,l,1,effect_option)
      if track_option == false then
        note_column:clear()
      end
    elseif column_nxt ~= nil and ln == 0
      and track_option == false
    then
      column_nxt.delay_value = dly
    else end
  else end
  return val
end



-- なんかButtonのPressedでエラーが出る。
-- 無名関数でLocal変数使いすぎた？謎や。
-- 試しに表現を簡易化
function nudge_plus ()
  local s,p,t,l,c = pos2()
  if renoise.song().selected_note_column_index ==0 then return
  end        
  nudge_it (s,p,t,l,c,false,1,nudge_effect_option)
end

function nudge_minus ()
  if renoise.song().selected_note_column_index ==0 then return
  end        
  local s,p,t,l,c = pos2()  
  nudge_it (s,p,t,l,c,false,-1,nudge_effect_option)
end


-- For PatternTrack Nudge。

-- Nudge_itの,TrackOption==true専用カスタム版。
-- 隣TrackへNoteを移す。
function nudge_to_track (s,p,t,l,c,effect_option,direction)
  local s_nxt,p_nxt,t_nxt,l_nxt,c_nxt
  local column_nxt
  local note_column =
  renoise.song().patterns[p].tracks[t].lines[l].note_columns[c]
  local ini_dly = note_column.delay_value
  local ln,dly
  local val = get_val_only (note_column,tags[1],tags[2])
  
  if nudge_ignore_option == true and already_nudged_maybe (s,t,l,c) == true
  then val = nil
  end
  if val ~=nil then val = direction*val end  
  -- ex.:delay2delay(nil,0,10)-> 0,10
  ln,dly = delay2delay (val,track_delay,ini_dly)
  s_nxt,p_nxt,t_nxt,l_nxt,c_nxt =
  next_pos (ln,s,p,t,l,c,true)
  -- パターン飛び出る奴はTrack移さない
  if (s_nxt ~= nil) and (s_nxt ~= s) then
    t_nxt = t_nxt -1
  end
  -- val~=nilだけど、端っこでs_nxtが存在しなかったりなど。例外。
  if s_nxt == nil then
    s_nxt,p_nxt,t_nxt,l_nxt,c_nxt = next_pos (0,s,p,t,l,c,true)
  end
  column_nxt =
  renoise.song().patterns[p_nxt].tracks[t_nxt]
  .lines[l_nxt].note_columns[c_nxt]

  column_nxt:copy_from(note_column)
  column_nxt.delay_value = dly
  note_column:clear()

    -- nudge_effect を追加。
  if effect_option == true  then
    nudge_effect (s_nxt,p_nxt,t_nxt,l_nxt,1,s,p,t,l,1,true)
  else
    nudge_effect (s_nxt,p_nxt,t_nxt,l,1,s,p,t,l,1,true)
  end
  
  if renoise.song().tracks[t_nxt].visible_note_columns < c_nxt
  then renoise.song().tracks[t_nxt].visible_note_columns = c_nxt
  end
  return val
end


function nudge_pattern_track (direction)
  local time1 = os.clock()  
  local drc
  if type(direction) ~= 'number' then drc = 1
  else drc = direction
  end
  if renoise.song().selected_note_column_index ==0 then return
  end
  --[[
  remove_note_notify() ; remove_notifi_selected_sample ()
  renoise.tool().app_idle_observable:remove_notifier(link_vb_to_globals)
  --]]
  
  local s,p,t,l,c = pos2 ()
  local init_vis = renoise.song().tracks[t].visible_note_columns
  renoise.song():insert_track_at(t+1)

  for pos,column in
  renoise.song().pattern_iterator
  :note_columns_in_pattern_track(p,t,true)
  do
    if column.is_empty == false
    then
      local val = nudge_to_track (s,p,t,pos.line,pos.column,nudge_effect_option,drc)
    end    
  end
  
  for pos,column in
  renoise.song().pattern_iterator
  :effect_columns_in_pattern_track(p,t,true)
  do
    if column.is_empty == false
    then
      nudge_effect (s,p,t+1,l,1,s,p,t,l,1,true)
    end
  end
  
  local t_nxt_vis = renoise.song().tracks[t+1].visible_note_columns
  local t_nxt_vis_e = renoise.song().tracks[t+1].visible_effect_columns  
  renoise.song().tracks[t].visible_note_columns = t_nxt_vis
  renoise.song().tracks[t].visible_effect_columns = t_nxt_vis_e

  for pos,column in
  renoise.song().pattern_iterator
  :note_columns_in_pattern_track(p,t+1,true)
  do
    if column.is_empty == false
    then
     renoise.song().patterns[p].tracks[t].lines[pos.line].note_columns[pos.column]
      :copy_from(column)      
    end    
  end

  for pos,column in
  renoise.song().pattern_iterator
  :effect_columns_in_pattern_track(p,t+1,true)
  do
    if column.is_empty == false
    then
     renoise.song().patterns[p].tracks[t].lines[pos.line].effect_columns[pos.column]
      :copy_from(column)      
    end    
  end  
  
  renoise.song():delete_track_at(t+1)
  renoise.song().selected_sequence_index = s
  renoise.song().selected_track_index = t
  renoise.song().selected_line_index = l
  if c > t_nxt_vis then
    c = t_nxt_vis
  end
  renoise.song().selected_note_column_index = c
  --[[
  link_active_window ()
  add_vb_notify()
  --]]
  local time2 = os.clock()
  print(time2-time1)
end

-- For Minus PatternTrack Nudge。
function minus_nudge_pattern_track ()
  nudge_pattern_track (-1)
end

-- undo
function undo()
  if renoise.song():can_undo() then
    renoise.song():undo()
  end
end

-- Functions for Notifiers

-- notifiers for selection_start in sample editor
function link_selection_val()
  local val = selection_start_2_ms_minus ()
  vb.views.id_val_from_selection_start.text =
  tostring(val)
end
  
function add_notifi_selection ()
  local buf =
  renoise.song().selected_sample.sample_buffer
  if not(buf.selection_start_observable:has_notifier(
    link_selection_val))
  then
    buf.selection_start_observable:add_notifier(
      link_selection_val)
  end
end
function remove_notifi_selection ()
  local buf =
  renoise.song().selected_sample.sample_buffer
  if (buf.selection_start_observable:has_notifier(
    link_selection_val))
  then
    buf.selection_start_observable:remove_notifier(
      link_selection_val)
  end
end


-- notifiers for selected_sample_index_observable
function link_selected_sample_idx_etc ()
  sample_idx =
  renoise.song().selected_sample_index
  sample_name = renoise.song().selected_sample.name
  sample_val = get_val_from_name (sample_name ,tags[1],tags[2])
  instrument_idx = renoise.song().selected_instrument_index
end

function add_notifi_selected_sample ()
  if not(renoise.tool().app_idle_observable:has_notifier(
    link_selected_sample_idx_etc))
  then
    renoise.tool().app_idle_observable:add_notifier(
      link_selected_sample_idx_etc)
  end
end
function remove_notifi_selected_sample ()
  if (renoise.tool().app_idle_observable:has_notifier(
    link_selected_sample_idx_etc))
  then
    renoise.tool().app_idle_observable:remove_notifier(
      link_selected_sample_idx_etc)
  end
end


-- notifiers for note in pattern editor
function watch_pos ()
  s_i,p_i,t_i,l_i,c_i = pos2() 
end

function catch_note ()
  local note_obj =
    renoise.song().selected_note_column
  if note_obj ~=nil then
    sample_val,sample_name,sample_idx,instrument_idx,delay_val =
    get_tagged_val_etc_from_note(note_obj,tags[1],tags[2])
  else end
end

function add_note_notify()
  if not (renoise.tool().app_idle_observable:has_notifier(watch_pos))
  then 
    renoise.tool().app_idle_observable:add_notifier(watch_pos)
    renoise.tool().app_idle_observable:add_notifier(catch_note)
  end
end

function remove_note_notify()
  if (renoise.tool().app_idle_observable:has_notifier(watch_pos)) 
  then
    renoise.tool().app_idle_observable:remove_notifier(watch_pos)
    renoise.tool().app_idle_observable:remove_notifier(catch_note)
  end
end


-- notifiers for linking vb objects to global values
-- delay値はstring.formatで、１６進数にradix conversion。
function link_vb_to_globals ()
  vb.views.id_info1.text = (tostring(sample_name))
  vb.views.id_info2.text = (tostring(sample_val).." "..tags[2])
  local l,d =delay2delay (sample_val,track_delay,delay_val)
  vb.views.id_info3.text = (tostring(l).." . "
    ..string.upper(string.format("%x",tostring(d))))
end
function add_vb_notify()
  if not (renoise.tool().app_idle_observable:has_notifier(link_vb_to_globals))
  then 
    renoise.tool().app_idle_observable:add_notifier(link_vb_to_globals)
  end
end
function remove_vb_notify()
  if (renoise.tool().app_idle_observable:has_notifier(link_vb_to_globals))
  then 
    renoise.tool().app_idle_observable:remove_notifier(link_vb_to_globals)
    vb.views.id_info1 = nil
    vb.views.id_info2 = nil
    vb.views.id_info3 = nil   
    vb.views.id_val_from_selection_start = nil
  end
end


-- notifiers for active window
function link_active_window ()
  local active = renoise.app().window.active_middle_frame
  current_window = active
  if active == ptn_edtr
  then add_note_notify() ; remove_notifi_selected_sample ()
     vb.views.id_val_from_selection_start.text = ""
  elseif active == smpl_edtr
  then remove_note_notify() ; add_notifi_selected_sample ()
  else remove_note_notify() ; remove_notifi_selected_sample ()
  end
end

function add_window_notify()
  if not (renoise.app().window.active_middle_frame_observable:
    has_notifier(link_active_window))
  then 
    renoise.app().window.active_middle_frame_observable:
    add_notifier(link_active_window)
  end
end
function remove_window_notify()
  if (renoise.app().window.active_middle_frame_observable:
    has_notifier(link_active_window))
  then 
    renoise.app().window.active_middle_frame_observable:
    remove_notifier(link_active_window)
  end
end


